#!/bin/csh -f

## Set Debug Parameter 
# DEBUG=1; erase all temporary files 
#       0; leave all temporary files 
set DEBUG=1 
 
## Set Pixel Scale (Not Used)
#set PIXSCALE=0.105  

## Parameters for Photometry
# Radius within which detect object to do photometry
set PHOTRADIUS=100
# Radius of annulus to measure sky
set PHOTAN=10
# Width of annulus to measure sky
set PHOTDAN=10

if ( $#argv != 3 ) then
    echo "Usage : "$0" (FITS data table) (# of dithering point) (Header Name)"
    exit 1
endif

set FTABLE=$1
set NPOINT=$2
set HEADER=`gawk -v hd=$3 'BEGIN{gsub(".fits","",hd);gsub(".imh","",hd);print(hd)}'`
set OUTIM=$HEADER".fits"

set PHOTTABLE=RESULTS/phot.$HEADER
set OFFSETTABLE=RESULTS/offset.$HEADER

set MLCOMBTABLE=mlcomb.tbl

set NFRAME=`gawk -v np=$NPOINT 'END{print(NR/np)}' $FTABLE`
echo NFRAME=$NFRAME

set DITH=20
@ DITH2 = $DITH * 2

set TMPFILELIST=tmpfile.list
if( -f $TMPFILELIST ) then
  \rm $TMPFILELIST
endif

set CLLOG=log/cl`date +%y%m%d%H%M%S`.log
set CLCOM=cl.com

echo $CLCOM >> $TMPFILELIST

if( !(-e log) ) then
  mkdir log
endif
if( -f $CLLOG ) then
  rm $CLLOG
endif

echo "IRAF Log of $0 $1 $2 $3" > $CLLOG

echo "IRAF log will be written in $CLLOG"
echo "### Erase Objects ###"
echo "### Erase Objects ###" >> $CLLOG
echo "imdel @mldata.tbl" > $CLCOM
echo "logout" >> $CLCOM 
cat $CLCOM | cl >> $CLLOG

set i=1
while($i <= $NPOINT)
  echo "# Erasing Objects in Position $i #"
  gawk -v p=$i -v f=$NFRAME '{if(NR>(p-1)*f && NR<=p*f) print $0}' ldata.tbl > ldata.tmp.tbl
  sed 's/ldata/mldata/g' ldata.tmp.tbl > mldata.tmp.tbl
  echo "eraseobj @ldata.tmp.tbl @mldata.tmp.tbl MASK/$HEADER."`gawk -v n=$i 'BEGIN{printf("%03d\n",n)}'`".tbl" > $CLCOM
  echo "logout" >> $CLCOM 
  cat $CLCOM | cl >>& $CLLOG
  @ i += 1
end
echo ldata.tmp.tbl >> $TMPFILELIST
echo mldata.tmp.tbl >> $TMPFILELIST

echo "# Adding Sky Again #"
echo "imdel @ldata.tbl" > $CLCOM
echo "imcopy @mldata.tbl  @ldata.tbl" >> $CLCOM
echo "imarith @ldata.1.tbl + RESULTS/$HEADER.sky1.fits @ldata.1.tbl" >> $CLCOM
if ( $NFRAME > 1 ) then
  echo "imarith @ldata.2.tbl + RESULTS/$HEADER.sky2.fits @ldata.2.tbl" >> $CLCOM
endif
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG

echo "### Re-Create Sky Frames ###"
echo "### Re-Create Sky Frames ###" >> $CLLOG
awk '{s+=$1;i++;}END{print(s/i)}' level.tbl > tmp.txt
echo tmp.txt >> $TMPFILELIST
echo "imdel noobjsky1,noobjsky2" > $CLCOM
echo "imdel noobjsky*.imh" >> $CLCOM
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
echo "# Running mksubgr #"
echo "del bkg1.tbl" >> $CLCOM
gawk -v nf=1 '{print("mksubgr @ldata.1.tbl noobjsky1 1 "nf" "$1" 257 768 257 768 bkg1.tbl")}' tmp.txt > $CLCOM
echo bkg1.tbl >> $TMPFILELIST
if ( $NFRAME > 1 ) then 
  echo "del bkg2.tbl" >> $CLCOM
  gawk -v nf=$NFRAME '{print("mksubgr @ldata.2.tbl noobjsky2 1 "nf-1" "$1" 257 768 257 768 bkg2.tbl")}' tmp.txt >> $CLCOM
  echo bkg2.tbl >> $TMPFILELIST
endif
echo "logout" >> $CLCOM 
cat $CLCOM | cl >>& $CLLOG

echo "### Re-Reduce Frames ###"
echo "### Re-Reduce Frames ###" >> $CLLOG

echo "# Divide Raw Frames by Flat #"  
echo "# Divide Raw Frames by Flat #" >> $CLLOG
echo "imdel @ldata.tbl" > $CLCOM
echo "imarith @"$FTABLE" / ./flat.fits @ldata.tbl" >> $CLCOM
#echo "del level.tbl" >> $CLCOM
#echo "imstatistics @pldata.1.tbl field=mean fo- > level.tbl" >> $CLCOM
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG

echo "# Adjust Offset in Quad 2 #"  
echo "# Adjust Offset in Quad 2 #" >> $CLLOG
echo "offsetrej @ldata.tbl" > $CLCOM
echo "logout" >> $CLCOM  
cat $CLCOM | cl >>& $CLLOG

awk '{s+=$1;i++;}END{print(s/i)}' level.tbl > tmp.txt
echo "imdel sky1,sky2" > $CLCOM
echo "imdel sky*.imh" >> $CLCOM
echo "del bkg1.tbl,bkg2.tbl" >> $CLCOM
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG

echo "# Running mksubgr #"
echo "# Running mksubgr #" >> $CLLOG
gawk -v nf=1 '{print("mksubgr @ldata.1.tbl sky1 1 "nf" "$1" 257 768 257 768 bkg1.tbl")}' tmp.txt > $CLCOM
if ( $NFRAME > 1 ) then 
  gawk -v nf=$NFRAME '{print("mksubgr @ldata.2.tbl sky2 1 "nf-1" "$1" 257 768 257 768 bkg2.tbl")}' tmp.txt >> $CLCOM
endif
echo "logout" >> $CLCOM  
cat $CLCOM | cl >>& $CLLOG

echo "### Subtract New Sky Frames ###"
echo "### Subtract New Sky Frames ###" >> $CLLOG
echo "imarith @ldata.1.tbl - noobjsky1 @ldata.1.tbl" > $CLCOM
if ( $NFRAME > 1 ) then 
 echo "imarith @ldata.2.tbl - noobjsky2 @ldata.2.tbl" >> $CLCOM
endif
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG


echo "### Modify ###"
echo "### Modify ###" >> $CLLOG
echo "imdel @mldata.tbl" > $CLCOM
echo "logout" >> $CLCOM 
cat $CLCOM | cl >>& $CLLOG
set i=1
while($i <= $NPOINT)
echo "# Adjusting Bias Offset in Position $i #"  
echo "# Adjusting Bias Offset in Position $i #" >> $CLLOG
  gawk -v p=$i -v f=$NFRAME '{if(NR>(p-1)*f && NR<=p*f) print $0}' ldata.tbl > ldata.tmp.tbl
  sed 's/ldata/mldata/g' ldata.tmp.tbl > mod_ldata.tmp.tbl
  echo "MASK/"$HEADER"."`gawk -v n=$i 'BEGIN{printf("%03d\n",n)}'`".tbl" > tmpmask.tbl
  echo "modify @ldata.tmp.tbl @mod_ldata.tmp.tbl @tmpmask.tbl" > $CLCOM
  echo "logout" >> $CLCOM 
  cat $CLCOM | cl >>& $CLLOG
  \rm tmpmask.tbl
  @ i += 1
end
echo mod_ldata.tmp.tbl >> $TMPFILELIST


echo "### Shift and Combine All Frames ###"
echo "### Shift and Combine All Frames ###" >> $CLLOG

echo "# Calculate Sigma for Single Frame #"
echo "# Calculate Sigma for Single Frame #" >> $CLLOG
echo "del tmp.sigma" > $CLCOM 
echo "imstatrej `sed -n 1p mldata.tbl`[300:600,300:600] 3 > tmp.sigma" >> $CLCOM
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
set SIGMA=`gawk '{if(NR==1)print($3*1.5)}' tmp.sigma` 
echo "  sigma=$SIGMA"
echo tmp.sigma >> $TMPFILELIST

echo $MLCOMBTABLE >> $TMPFILELIST
if ( -f $MLCOMBTABLE ) then
  \rm $MLCOMBTABLE
endif
set i=1
while($i <= $NPOINT)
echo "# Shift and Add at Position $i #"
echo "# Shift and Add at Position $i #" >> $CLLOG
  echo " # Object Detection #"
  echo " # Object Detection #" >> $CLLOG
  set PMLTABLE=mldata.$i.tbl
  echo $PMLTABLE >> $TMPFILELIST
  set PMLCOMB=RESULTS/$HEADER.$i.fits
  gawk -v i=$i -v nf=$NFRAME '{if(NR>nf*(i-1) && NR<=nf*i) print($0)}' mldata.tbl > $PMLTABLE
  echo "noao" > $CLCOM 
  echo "digiphot" >> $CLCOM 
  echo "apphot" >> $CLCOM
  foreach filename (`cat $PMLTABLE`) 
    set COORDFILE=`echo $filename | sed 's/.imh/.coo/g' | sed 's/.fits/.coo/g'`
    echo $COORDFILE >> $TMPFILELIST
    if ( -f $COORDFILE ) then
      \rm $COORDFILE 
    endif
    echo "imdel tmpmed" >> $CLCOM
    echo "median "$filename" tmpmed 3 3" >> $CLCOM
    echo "daofind tmpmed output="$COORDFILE" fwhmpsf=8 thresho=4 verify- sigma="$SIGMA" sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5" >> $CLCOM 
    echo "display $filename 1 fi+" >> $CLCOM
    echo "tvmark 1 $COORDFILE number+" >> $CLCOM
  end 
  echo "logout" >> $CLCOM
  cat $CLCOM | cl >>& $CLLOG
  \cp $COORDFILE tmp.coord
  grep -v "^#" tmp.coord > $COORDFILE
  \rm tmp.coord
  
  echo " # Pattern Matching #"
  echo " # Pattern Matching #" >> $CLLOG
  @ j = 1 
  foreach filename (`cat $PMLTABLE`)  
    set COORDFILE=`echo $filename | sed 's/.imh/.coo/g'| sed 's/.fits/.coo/g'`  
#    echo $COORDFILE $j 
    if ( $j == 1 ) then 
      set BASEFILE=$COORDFILE
      echo "0 0" > tmp.offset 
    else 
      ./matchpat $BASEFILE $COORDFILE 10 >> tmp.offset
    endif 
    @ j = $j + 1
  end
  echo " # Combine frames: Output is $PMLCOMB #"
  echo " # Combine frames: Output is $PMLCOMB #" >> $CLLOG
  echo "imdel $PMLCOMB" > $CLCOM
  echo "imcombine @$PMLTABLE $PMLCOMB combine=average reject=sigclip outtype=real lsigma=2 hsigma=2 offset=tmp.offset" >> $CLCOM
  echo "display $PMLCOMB 1 fi+" >> $CLCOM
  echo "logout" >> $CLCOM
  cat $CLCOM | cl >>& $CLLOG
  echo $PMLCOMB >> $MLCOMBTABLE
  @ i += 1
end
echo tmp.offset >> $TMPFILELIST



echo "### Pattern Matching of Coadded Frames ###"
echo "### Pattern Matching of Coadded Frames ###" >> $CLLOG 
echo "# Calculate Sigma for Single Frame #"
echo "# Calculate Sigma for Single Frame #" >> $CLLOG 
echo "del tmp.sigma" > $CLCOM 
echo "imstatrej `sed -n 1p $MLCOMBTABLE`[300:600,300:600] 3 > tmp.sigma" >> $CLCOM
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
set SIGMA=`gawk '{if(NR==1)print($3*1)}' tmp.sigma` 
echo "  sigma=$SIGMA" 

echo " # Object Detection #" 
echo " # Object Detection #" >> $CLLOG 
echo "noao" > $CLCOM
echo "digiphot" >> $CLCOM
echo "apphot" >> $CLCOM
foreach filename (`cat $MLCOMBTABLE`)  
  set COORDFILE=`echo $filename | sed 's/.imh/.coo/g' | sed 's/.fits/.coo/g'`  
  if( -f $COORDFILE ) then
    \rm $COORDFILE 
  endif
  echo "imdel tmpmed" >> $CLCOM
  echo "median "$filename" tmpmed 3 3" >> $CLCOM
  echo "daofind tmpmed output="$COORDFILE" fwhmpsf=8 thresho=4 verify- sigma="$SIGMA" sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5" >> $CLCOM 
  echo "display $filename 1 fi+" >> $CLCOM
  echo "tvmark 1 $COORDFILE number+" >> $CLCOM
end  
echo "logout" >> $CLCOM 
cat $CLCOM | cl >>& $CLLOG

@ i = 1 
foreach filename (`cat $MLCOMBTABLE`)  
  set COORDFILE=`echo $filename | sed 's/.imh/.coo/g'| sed 's/.fits/.coo/g'`  
  set COORDFILE2=`echo $filename | sed 's/.imh/2.coo/g' | sed 's/.fits/2.coo/g'`  
  set XOFFSET=`sed -n ${i}p $OFFSETTABLE | gawk '{print($1)}'`
  set YOFFSET=`sed -n ${i}p $OFFSETTABLE | gawk '{print($2)}'`
  grep -v "^#" $COORDFILE | gawk -v dx=$XOFFSET -v dy=$YOFFSET '{print($1+dx,$2+dy)}' > $COORDFILE2
  echo $COORDFILE >> $TMPFILELIST
  echo $COORDFILE2 >> $TMPFILELIST
  @ i = $i + 1
end

echo "# Pattern Matching #"
echo "# Pattern Matching #" >> $CLLOG
@ i = 1 
foreach filename (`cat $MLCOMBTABLE`)   
  set COORDFILE2=`echo $filename | sed 's/.imh/2.coo/g' | sed 's/.fits/2.coo/g'`  
#  echo $COORDFILE2 $i
  if ( $i == 1 ) then
    set BASEFILE=$COORDFILE2
    echo "0 0" > tmp.offset
  else
    ./matchpat $BASEFILE $COORDFILE2 5 >> tmp.offset
  endif
  @ i = $i + 1
end
paste $OFFSETTABLE tmp.offset | gawk '{print($1+$3,$2+$4)}' > tmp.offset2
echo tmp.offset2 >> $TMPFILELIST

echo "### Shift and Add ###"
echo "### Shift and Add ###" >> $CLLOG
echo "imdel tmp.tot.imh" > $CLCOM
echo "imcombine @$MLCOMBTABLE tmp.tot combine=median reject=sigclip outtype=real lsigma=2 hsigma=2 offset=tmp.offset2" >> $CLCOM
echo "display tmp.tot 1 fi+" >> $CLCOM
echo "imdel RESULTS/$OUTIM" >> $CLCOM 
echo "imcopy tmp.tot RESULTS/$OUTIM" >> $CLCOM 
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
\cp tmp.offset2 $OFFSETTABLE

echo "### $0 Finished ###"
echo "### $0 Finished ###" >> $CLLOG
echo "Final frame is saved as RESULTS/"$HEADER".fits"
echo "Single frames are saved as;"
cat $MLCOMBTABLE
echo "Offset pattern file is saved as "$OFFSETTABLE
echo "Offset pattern is : " 
cat $OFFSETTABLE
echo "IRAF log written in $CLLOG"
if ( $DEBUG == 1 ) then
echo "### Erase Temporary Files ###"
echo "### Erase Temporary Files ###" >> $CLLOG
  foreach filename (`cat $TMPFILELIST`)
#    if ( -f $filename ) then
      \rm $filename
#    endif
  end
#  gawk '{print("\rm "$0)}' $TMPFILELIST | csh
endif
